{==============================================================================}
{
  TRichView demo: how to store images directly in HTML.
  This demo is for Delphi XE7, because it uses TNetEncoding.Base64 from
  System.NetEncoding
}
{==============================================================================}

unit Unit1;

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, RVStyle, ComCtrls,
  NetEncoding, Jpeg, GifImg, PngImage,
  RVGifAnimate2007, RVGrHandler, RVFuncs,
  CRVData, CRVFData, RVTable, RVTypes, RVItem, RVScroll, RichView, RVEdit;

type
  TForm1 = class(TForm)
    RichView1: TRichViewEdit;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    Button1: TButton;
    StatusBar1: TStatusBar;
    Button2: TButton;
    SaveDialog1: TSaveDialog;
    RVStyle1: TRVStyle;
    procedure Button1Click(Sender: TObject);
    procedure RichView1RVMouseMove(Sender: TObject; id: Integer);
    procedure RichView1Jump(Sender: TObject; id: Integer);
    procedure Button2Click(Sender: TObject);
    procedure RichView1SaveImage2(Sender: TCustomRichView; Graphic: TGraphic;
      SaveFormat: TRVSaveFormat; const Path, ImagePrefix: TRVUnicodeString;
      var ImageSaveNo: Integer; var Location: TRVUnicodeString;
      var DoDefault: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}



{============================== RVF loading ===================================}
procedure TForm1.Button1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    if not RichView1.LoadRVF(OpenDialog1.FileName) then
      Application.MessageBox('Error Loading File', nil, MB_OK);
    RichView1.Format;
  end;
end;
{============================ Hypertext testing ===============================}
procedure TForm1.RichView1RVMouseMove(Sender: TObject; id: Integer);
var RVData: TCustomRVFormattedData;
    ItemNo: Integer;
begin
  if id=-1 then
    StatusBar1.SimpleText := ''
  else begin
    RichView1.GetJumpPointLocation(id, RVData, ItemNo);
    StatusBar1.SimpleText := RVData.GetItemTag(ItemNo);
  end;
end;
{------------------------------------------------------------------------------}
procedure TForm1.RichView1Jump(Sender: TObject; id: Integer);
var RVData: TCustomRVFormattedData;
    ItemNo: Integer;
begin
  RichView1.GetJumpPointLocation(id, RVData, ItemNo);
  StatusBar1.SimpleText := RVData.GetItemTag(ItemNo);
  Application.MessageBox(PChar(StatusBar1.SimpleText),'Click', 0);
end;
{============================ SAVING TO HTML ==================================}
procedure TForm1.Button2Click(Sender: TObject);
var r: Boolean;
begin
  if SaveDialog1.Execute then begin
    Screen.Cursor := crHourglass;
    case SaveDialog1.FilterIndex of
      1:
        r := RichView1.SaveHTML(SaveDialog1.FileName,
          'Demo File','', [rvsoUTF8]);
      2:
        r := RichView1.SaveHTMLEx(SaveDialog1.FileName,
          'Demo File','', '','','', [rvsoUTF8]);
      else
        r := False;
    end;
    Screen.Cursor := crDefault;
    if not r then
      Application.MessageBox('Error during saving', 'Error', 0);
  end;
end;
{------------------------------------------------------------------------------}
function GetImageContentType(gr: TGraphic): TRVUnicodeString;
begin
  case RVGraphicHandler.GetGraphicType(gr) of
    rvgtBitmap:
      Result := 'image/bmp';
    rvgtIcon:
      Result := 'image/vnd.microsoft.icon';
    rvgtJPEG:
      Result := 'image/jpeg';
    rvgtPNG:
      Result := 'image/png';
    rvgtGIF:
      Result := 'image/gif';
    rvgtTIFF:
      Result := 'image/tiff';
    else
      Result := 'application/octet-stream';
  end;
end;
{------------------------------------------------------------------------------}
// If Gr is a graphic format appropriate for HTML, returns Gr
// Otherwise, returns Gr converted to Png
function GetHTMLGraphic(Gr: TGraphic): TGraphic;
var
  Bmp: TBitmap;
begin
  if RVGraphicHandler.IsHTMLGraphic(Gr) then
  begin
    Result := Gr;
    exit;
  end;
  Result := nil;
  try
    if RVGraphicHandler.GetGraphicType(Gr) = rvgtIcon then
      Result := RVGraphicHandler.IconToPng(Gr)
    else
    begin
      Result := RVGraphicHandler.CreateGraphicByType(rvgtPNG);
      try
        Result.Assign(Gr);
      except
        Bmp := RVGraphicHandler.ToBitmap(Gr);
        try
          Result.Assign(Bmp);
        finally
          Bmp.Free;
        end;
      end;
    end;
  except
    Result.Free;
    Result := Gr;
  end;
end;
{------------------------------------------------------------------------------}
procedure TForm1.RichView1SaveImage2(Sender: TCustomRichView; Graphic: TGraphic;
  SaveFormat: TRVSaveFormat; const Path, ImagePrefix: TRVUnicodeString;
  var ImageSaveNo: Integer; var Location: TRVUnicodeString;
  var DoDefault: Boolean);
var
  StreamIn, StreamOut: TMemoryStream;
  Base64Data: TRVAnsiString;
  GraphicToSave: TGraphic;
  i: Integer;
begin
  if SaveFormat <> rvsfHTML then
    exit;
  StreamIn := TMemoryStream.Create;
  StreamOut := TMemoryStream.Create;
  GraphicToSave := GetHTMLGraphic(Graphic);
  try
    GraphicToSave.SaveToStream(StreamIn);
    StreamIn.Position := 0;
    TNetEncoding.Base64.Encode(StreamIn, StreamOut);
    StreamOut.Position := 0;
    SetLength(Base64Data, StreamOut.Size);
    StreamOut.ReadBuffer(PRVAnsiChar(Base64Data)^, StreamOut.Size);
    for i := Length(Base64Data) downto 1 do
      if Base64Data[i] in [#13, #10] then
        Delete(Base64Data, i, 1);
    Location := 'data:' + GetImageContentType(GraphicToSave) + ';base64,' +
      String(Base64Data);
    DoDefault := False;
  finally
    StreamIn.Free;
    StreamOut.Free;
    if GraphicToSave <> Graphic then
      GraphicToSave.Free;
  end;
end;


end.
